from __future__ import absolute_import, division, print_function, unicode_literals

from os import path
try:
    from urllib.parse import urlparse, urlunparse, ParseResult
except ImportError:
    from urlparse import urlparse, urlunparse, ParseResult

from nib import Processor, after, Document

@after
class PaginationProcessor(Processor):
    def process(self, documents, resources):
        new_documents = list()
        for document in documents:
            if document.get('paginate', False) and 'pages' in document:
                if isinstance(document['pages'], list):
                    per_page = document['per_page']
                    sort_by = document['sort_by']
                    reverse = document['reverse']

                    unknowns = filter(lambda d: sort_by not in d,
                                      document['pages'])
                    knowns = filter(lambda d: sort_by in d,
                                    document['pages'])

                    pages = sorted(knowns,
                                   key=lambda d: d[sort_by],
                                   reverse=reverse)
                    pages.extend(unknowns)

                    document['pages'] = pages[:per_page]
                    pages = pages[per_page:]

                    last = document
                    counter = 1
                    while len(pages):
                        doc = document.clone()
                        doc.path += str(counter)

                        p, e = path.splitext(doc.uri)
                        doc.uri = p + str(counter) + e

                        doc['pages'] = pages[:per_page]
                        pages = pages[per_page:]

                        last['next'] = doc
                        doc['previous'] = last
                        doc.pop('next', None)

                        new_documents.append(doc)

                        last = doc
                        counter += 1

        documents.extend(new_documents)

        return documents, resources




from __future__ import absolute_import, division, print_function, unicode_literals

from nib import Document, Processor, after

@after
class IndexPages(Processor):
    def process(self, documents, resources):
        indexes = {}

        for document in documents:
            if 'parent' in document:
                index = document['parent']

                if index not in indexes:
                    indexes[index] = list()

                indexes[index].append(document)

        for document in documents:
            if document.path in indexes:
                document['pages'] = indexes[document.path]
                if document['template'] == self.options['defaults']['template']:
                    document['template'] = 'list.html'
                del indexes[document.path]

        for index_path in indexes:
            title = ' '.join(index_path.split('/')).title()
            document = Document(path=index_path,
                                uri=index_path,
                                title=title,
                                content='',
                                short='',
                                template='list.html',
                                pages=indexes[index_path])
            print(document.uri)
            documents.append(document)

        return documents, resources



from __future__ import absolute_import, division, print_function, unicode_literals

import jinja2
from jinja2 import Environment, FileSystemLoader, Template
from os import path
import time

jinja_filters = {}

def jinja(name):
    def decorator(f):
        jinja_filters[name] = f
        return f
    return decorator

class Render(object):
    def __init__(self, options, documents):
        self.options = options
        self.documents = documents

        self.loader = FileSystemLoader(path.abspath(options['template_path']))
        self.env = Environment(loader=self.loader)
        for name in jinja_filters:
            self.env.filters[name] = jinja_filters[name]

        self.site = dict(options['site'], documents=documents)
        self.now = time.time()

    def render_content(self, document):
        params = {
            'now': self.now,
            'site': self.site,
            'page': document,
        }
        params.update(document)

        document.short = self.env.from_string(document.short).render(**params)
        document.content = self.env.from_string(document.content).render(**params)

    def render_template(self, document):
        if 'template' in document:
            template = self.env.get_template(document['template'])

            params = {
                'now': self.now,
                'site': self.options['site'],
                'page': document,
                'content': document.content,
                'short': document.short,
            }
            params.update(document)

            return template.render(**params)

        else:
            return document.content



from __future__ import absolute_import, division, print_function, unicode_literals

import re
import yaml

document_marker = re.compile(r'^---\s*$', re.M)

try:
    from yaml import CSafeLoader as SafeLoader
    from yaml import CSafeDumper as SafeDumper
except ImportError:
    from yaml import SafeLoader, SafeDumper

def load(filename, supplement=False):
    with open(filename) as f:
        content = f.read()

    documents = document_marker.split(content, 2)

    try:
        data_document = documents.pop(0)
        data = yaml.load(data_document, Loader=SafeLoader)

    except yaml.YAMLError:
        if supplement:
            data = {}
            documents.insert(0, data_document)
        else:
            raise

    if supplement:
        return data, documents
    else:
        return data

def save(filename, data, supplement=None):
    output = yaml.dump(dict(data), Dumper=SafeDumper, indent=4,
                       default_flow_style=False)
    with open(filename, 'w') as f:
        f.write(output)

        if supplement:
            f.write('---\n')
            f.write(supplement)




from __future__ import absolute_import, division, print_function, unicode_literals

from os import path
import sys

import nib

default_config = path.join(nib.cwd, 'defaults.nib')

def merge(dest, source):
    """In-place, recursive merge of two dictionaries."""
    for key in source:
        if key in dest:
            if isinstance(dest[key], dict) and isinstance(source[key], dict):
                merge(dest[key], source[key])
                continue

        dest[key] = source[key]

    return dest

class Config(dict):
    def __init__(self, filename=None):
        values = nib.yaml.load(default_config)

        if filename is not None:
            if path.isfile(filename):
                overrides = nib.yaml.load(filename)
                merge(values, overrides)
            else:
                sys.stderr.write('Warning: no site config found at "{}"\n'.format(filename))

        dict.__init__(self, values)



from __future__ import absolute_import, division, print_function, unicode_literals

from os import path
import sh
from nib import Processor, resource

@resource('.less')
class LessCSSProcessor(Processor):
    def resource(self, resource):
        filepath = path.join(self.options['resource_path'],
                             resource.path + resource.extension)
        resource.content = bytearray(str(sh.lessc(filepath)), 'utf-8')
        resource.extension = '.css'

        return resource



from __future__ import absolute_import, division, print_function, unicode_literals

version = '0.5.4'

from os import path
cwd = path.abspath(path.dirname(__file__))

_instance = None
def instance(new=None):
    global _instance
    if new:
        _instance = new
    return _instance

from . import yaml
from .config import Config
from .document import Document
from .resource import Resource
from .render import Render, jinja
from .processor import Processor
from .processor import before, after, document, resource, markup, render
from .build import Build

from . import plugins




from __future__ import absolute_import, division, print_function, unicode_literals

from nib import Processor, render

@render
class PagesProcessor(Processor):
    def process(self, documents, resources):
        for document in documents:
            if 'pages' in document:
                if type(document['pages']) == str:
                    code = compile(document['pages'], '<string>', 'eval')
                    func = lambda page: eval(code)

                    subpages = list(filter(func, documents))
                    document['pages'] = subpages

        return documents, resources



from __future__ import absolute_import, division, print_function, unicode_literals

import os.path

class Resource(object):
    def __init__(self, path=None, content=None):
        self.path, self.extension = os.path.splitext(path)
        self.content = content

    @classmethod
    def from_file(cls, path, options=None):
        with open(path, 'rb') as f:
            content = f.read()

        if options:
            path = os.path.relpath(path, options['resource_path'])

        return Resource(path=path,
                        content=content,
                        )



from setuptools import setup

import os
from os import path
import shutil

if path.isfile('README.md'):
    shutil.copyfile('README.md', 'README')

if path.isdir('sample'):
  if path.exists('sample/site'):
      shutil.rmtree('sample/site')
  if path.exists('sample/config.nib'):
      os.unlink('sample/config.nib')
  shutil.make_archive('nib/sample', 'zip', 'sample')

setup(name='Nib',
      description='Static Site Generator',
      version='0.5.4',
      author='John Reese',
      author_email='john@noswap.com',
      url='https://github.com/jreese/nib',
      classifiers=['License :: OSI Approved :: MIT License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3.2',
                   'Programming Language :: Python :: 3.3',
                   'Topic :: Utilities',
                   'Development Status :: 4 - Beta',
                   ],
      license='MIT License',
      install_requires=['sh>=1.0',
                        'Jinja2>=2.6',
                        'Markdown>=2.2.0',
                        'PyYAML>=3.10',
                        ],
      requires=['sh (>=1.0)',
                'Jinja2 (>=2.6)',
                'Markdown (>=2.2.0)',
                'PyYAML (>=3.10)',
                ],
      packages=['nib', 'nib.plugins'],
      package_data={'nib': ['defaults.nib', 'sample.zip']},
      scripts=['bin/nib'],
      )
